package com.example.elbuenopeso.ui.add

import android.app.Activity
import android.content.ContentResolver
import android.content.Context
import android.content.ContextWrapper
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.provider.MediaStore
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageButton
import android.widget.ImageView
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.Fragment
import com.example.elbuenopeso.R
import com.example.elbuenopeso.database.DBManager
import com.example.elbuenopeso.databinding.FragmentAddBinding
import com.example.elbuenopeso.models.AdModel
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStream
import java.io.OutputStream
import java.lang.NumberFormatException


class AddFragment : Fragment() {

    private var _binding: FragmentAddBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    private lateinit var cameraButton: ImageButton
    private lateinit var image: ImageView
    private lateinit var sendButton: Button
    private var filePath: String = ""

    private val galleryActivityResultLauncher: ActivityResultLauncher<Intent> =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            result -> run {
                    if (result.resultCode == Activity.RESULT_OK) {
                        val image: Uri = result?.data?.data as Uri

                        val cw = ContextWrapper(requireContext())
                        val directory: File = cw.getDir("imageDir", Context.MODE_PRIVATE)
                        val tsLong = System.currentTimeMillis() / 1000
                        val ts = tsLong.toString()
                        val file = File(directory, "$ts.jpg")

                        val in_stream: InputStream? = requireContext().contentResolver.openInputStream(image)
                        val out_stream: OutputStream = FileOutputStream(file)
                        val buf = ByteArray(1024)
                        var len: Int
                        while (in_stream!!.read(buf).also { len = it } > 0) {
                            out_stream.write(buf, 0, len)
                        }
                        out_stream.flush()
                        out_stream.close()
                        in_stream.close()

                        this.filePath = file.toString()
                        this.image.setImageURI(image)
                    }
            }
        }

    @Suppress("DEPRECATION")
    private val cameraActivityResultLauncher: ActivityResultLauncher<Intent> =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            result -> if (result.resultCode == Activity.RESULT_OK) {
                val data = result.data
                val bitmap = data!!.extras!!["data"] as Bitmap?
                val cw = ContextWrapper(requireContext())
                val directory: File = cw.getDir("imageDir", Context.MODE_PRIVATE)
                val tsLong = System.currentTimeMillis() / 1000
                val ts = tsLong.toString()
                val file = File(directory, "$ts.jpg")
                if (!file.exists()) {
                    Log.d("path", file.toString())
                    var fos: FileOutputStream? = null
                    try {
                        fos = FileOutputStream(file)
                        bitmap!!.compress(Bitmap.CompressFormat.JPEG, 100, fos)
                        fos.flush()
                        fos.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }
                }
                this.filePath = file.toString()
                this.image.setImageBitmap(bitmap)
            }
        }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentAddBinding.inflate(inflater, container, false)
        val root: View = binding.root
        image = binding.image
        cameraButton = binding.cameraButton
        sendButton = binding.sendButton

        // DB manager :
        val dbManager = DBManager.getDBManager(requireContext())
        dbManager!!.open()

        image.setOnClickListener {
            val gallery = Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI)
            galleryActivityResultLauncher.launch(gallery)
        }

        cameraButton.setOnClickListener {
            val camera = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            cameraActivityResultLauncher.launch(camera)
        }

        sendButton.setOnClickListener {
            if(binding.title.text.toString().isEmpty() || binding.price.text.toString().isEmpty()){
                return@setOnClickListener
            }

            Log.d(null, this.filePath)

            val ad = AdModel(
                binding.title.text.toString(),
                binding.address.text.toString(),
                this.filePath,
                prix = run {
                    try {
                        binding.price.text.toString().replace(',', '.').toDouble()
                    } catch (e: NumberFormatException) {
                        Double.NaN
                    }
                },
                binding.num.text.toString()
            )

            dbManager.insert(ad);

            image.setImageResource(R.drawable.baseline_add_24)
            binding.title.setText("")
            binding.address.setText("")
            binding.price.setText("")
            binding.num.setText("")
            this.filePath = ""

            // Je sais pas trop ce que c'est mais dans le doute je laisse ça là...
//            val intent: Intent = Intent(
//                this@AddFragment,
//                DbAdListViewActivity::class.java
//            )
//            startActivity(intent)
        }

        return root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}